/*
 File:		MoriarityController.m

 Description: 	This is the implementation for our "controller" class that 
 operates as the go-between for the view (the UI) and the model (the locate 
 database).  The point of this code is to show how you can wrap a UNIX task 
 in a nice Cocoa GUI, and get back the output (which you could then operate 
 on or parse further if you choose).

 Original:	http://developer.apples.com/samplecode/Moriarity/Moriarity.html
 
 Modifications:	This originally used the TaskWrapper object to run "locate" commands
    I modified it to instead show a ReadMe, License, run an install or uninstall script,
    and show the results (with some fancy text formatting).

 $Header: /usr/local/cvsrep/NoFrillsInstaller/MoriarityController.m,v 1.3 2004/10/04 12:08:11 kitzelh Exp $
 */


#import "MoriarityController.h"

@implementation MoriarityController

// I don't remember where I got this code.  Somewhere off of the net.
// It searches the given textView for a string and modifies the color
// of those characters, optionally making it a hyperlink.
- (void)hiliteAndActivateURLs:(NSTextView*)textView 
    scheme:(NSString*)scheme 
    color:(NSColor*)color
    link:(BOOL)link 
{
    NSTextStorage *textStorage = [textView textStorage];
    NSString *string = [textStorage string];
    NSRange searchRange = NSMakeRange(0, [string length]);
    NSRange foundRange;
    
    [textStorage beginEditing];
    do {
        foundRange=[string rangeOfString:scheme options:0 range:searchRange];
        if (foundRange.length > 0) {
            NSURL * theURL;
            NSDictionary *linkAttributes;
            NSRange endOfURLRange;
            
            searchRange.location = foundRange.location+foundRange.length;
            searchRange.length = [string length] - searchRange.location;
            
            endOfURLRange = [string rangeOfCharacterFromSet:
                [NSCharacterSet whitespaceAndNewlineCharacterSet]
                options:0 range:searchRange];
            
            if (endOfURLRange.length == 0) endOfURLRange.location = [string length]-1;
            foundRange.length = endOfURLRange.location-foundRange.location+1;
            theURL = [NSURL URLWithString:[string substringWithRange:foundRange]];
            if (link) {
                linkAttributes = [NSDictionary dictionaryWithObjectsAndKeys:
                    theURL, NSLinkAttributeName,
                    [NSNumber numberWithInt:NSSingleUnderlineStyle], NSUnderlineStyleAttributeName,
                    color, NSForegroundColorAttributeName,
                    NULL];
            } else {
                linkAttributes = [NSDictionary dictionaryWithObjectsAndKeys:
                    color, NSForegroundColorAttributeName,
                    NULL];
            }
                
            [textStorage addAttributes:linkAttributes range:foundRange];
        }
    } while (foundRange.length != 0);
    [textStorage endEditing];
}

// This is the workhorse part of the class.  Each phase sets what is displayed, what the buttons
// are named, and what they do.
-(void)doPhase:(int)whichPhase
{
    phase = whichPhase;
    switch (whichPhase) {
    case nfiPhaseReadme :
        [resultsTextField readRTFDFromFile:[[NSBundle mainBundle] pathForResource:@"ReadMe" ofType:@"rtf"]];
        [self hiliteAndActivateURLs:resultsTextField scheme:@"http://" color:[NSColor blueColor] link:YES];
        [self hiliteAndActivateURLs:resultsTextField scheme:@"mailto:" color:[NSColor blueColor] link:YES];
        [sleuthButton setTitle:NSLocalizedString(@"Install",@"Install Button")];
        [slinkButton setTitle:NSLocalizedString(@"Uninstall",@"Uninstall Button")];
        [slinkButton setEnabled:YES];
        [slinkButton setTransparent:NO];
        break;
    case nfiPhaseLicense :
        [resultsTextField readRTFDFromFile:[[NSBundle mainBundle] pathForResource:@"License" ofType:@"rtf"]];
        [sleuthButton setTitle:NSLocalizedString(@"Agree",@"Agree to License")];
        [slinkButton setTitle:NSLocalizedString(@"Back",@"Back Button")];
        [slinkButton setEnabled:YES];
        [slinkButton setTransparent:NO];
        break;
    case nfiPhaseInstall : {
        NSString *resources = [[NSBundle mainBundle] resourcePath];
        // If the task is still sitting around from the last run, release it
        if (theTask!=nil) [theTask release];
        [resultsTextField setString:@""];
        [sleuthButton setTitle:NSLocalizedString(@"Stop",@"Stop Process")];
        [slinkButton setTitle:@""];
        [slinkButton setEnabled:NO];
        [slinkButton setTransparent:YES];
        // Let's allocate memory for and initialize a new TaskWrapper object, passing
        // in ourselves as the controller for this TaskWrapper object, the path
        // to the command-line tool, and the contents of the text field that 
        // displays what the user wants to search on
        theTask=[[TaskWrapper alloc] 
            initWithController:self 
            arguments:[NSArray arrayWithObjects:
                [resources stringByAppendingString:@"/nfi_install"], 
                resources, nil
            ]
        ];
        // kick off the process asynchronously
        [theTask startProcess];
        }
        break;
    case nfiPhaseDone:
        [sleuthButton setTitle:NSLocalizedString(@"Quit",@"Quit Button")];
        [slinkButton setTitle:@""];
        [slinkButton setEnabled:NO];
        break;
    case nfiPhaseExit:
        [NSApp terminate:nil];
    case nfiPhaseUninstall : {
        NSString *resources = [[NSBundle mainBundle] resourcePath];
        // If the task is still sitting around from the last run, release it
        if (theTask!=nil) [theTask release];
        [resultsTextField setString:@""];
        [sleuthButton setTitle:NSLocalizedString(@"Stop",@"Stop Process")];
        [slinkButton setTitle:@""];
        [slinkButton setEnabled:NO];
        [slinkButton setTransparent:YES];
        // Let's allocate memory for and initialize a new TaskWrapper object, passing
        // in ourselves as the controller for this TaskWrapper object, the path
        // to the command-line tool, and the contents of the text field that 
        // displays what the user wants to search on
        theTask=[[TaskWrapper alloc] 
            initWithController:self 
            arguments:[NSArray arrayWithObjects:
                [resources stringByAppendingString:@"/nfi_uninstall"], 
                resources, nil
            ]
        ];
        // kick off the process asynchronously
        [theTask startProcess];
        }
        break;
    default :
        break;
    }
}

// This action kicks off the next phase if we aren't already running a task,
// or stops the current task if one is already running
- (IBAction)sleuth:(id)sender
{
    if (findRunning)
    {
        // This stops the task and calls our callback (-processFinished)
        [theTask stopProcess];
        // Release the memory for this wrapper object
        [theTask release];
        theTask=nil;
        [self doPhase:phase-1];
        return;
    }
    else
    {
        [self doPhase:phase+1];
    }
}

// This action kicks off a previous phase if we aren't already running a task,
// or stops the current task if one is already running
- (IBAction)slink:(id)sender
{
    if (findRunning)
    {
        // This stops the task and calls our callback (-processFinished)
        [theTask stopProcess];
        // Release the memory for this wrapper object
        [theTask release];
        theTask=nil;
        [self doPhase:nfiPhaseReadme];
        return;
    }
    else
    {
        if (phase==nfiPhaseReadme) [self doPhase:nfiPhaseUninstall];
        else if (phase == nfiPhaseLicense) [self doPhase:nfiPhaseReadme];
    }
}

// This callback is implemented as part of conforming to the ProcessController protocol.
// It will be called whenever there is output from the TaskWrapper.
- (void)appendOutput:(NSString *)output
{
    // add the string (a chunk of the results from locate) to the NSTextView's
    // backing store, in the form of an attributed string
    [[resultsTextField textStorage] appendAttributedString: [
        [[NSAttributedString alloc]
        initWithString: output
        attributes:[NSDictionary dictionaryWithObjectsAndKeys:
            [NSFont userFontOfSize:16], NSFontAttributeName,
            NULL
        ]
    ] autorelease]];
    // setup a selector to be called the next time through the event loop to scroll
    // the view to the just pasted text.  We don't want to scroll right now,
    // because of a bug in Mac OS X version 10.1 that causes scrolling in the context
    // of a text storage update to starve the app of events
    [self performSelector:@selector(scrollToVisible:) withObject:nil afterDelay:0.0];
}

// This routine is called after adding new results to the text view's backing store.
// We now need to scroll the NSScrollView in which the NSTextView sits to the part
// that we just added at the end
- (void)scrollToVisible:(id)ignore {
    [resultsTextField scrollRangeToVisible:NSMakeRange([[resultsTextField string] length], 0)];
}

// A callback that gets called when a TaskWrapper is launched, allowing us to do any setup
// that is needed from the app side.  This method is implemented as a part of conforming
// to the ProcessController protocol.
- (void)processStarted
{
    findRunning=YES;
}

// A callback that gets called when a TaskWrapper is completed, allowing us to do any cleanup
// that is needed from the app side.  This method is implemented as a part of conforming
// to the ProcessController protocol.
- (void)processFinished
{
    findRunning=NO;
    [self hiliteAndActivateURLs:resultsTextField scheme:@"http://" color:[NSColor blueColor] link:YES];
    [self hiliteAndActivateURLs:resultsTextField scheme:@"mailto:" color:[NSColor blueColor] link:YES];
    [self hiliteAndActivateURLs:resultsTextField scheme:@"Error!" color:[NSColor redColor] link:NO];
    [self hiliteAndActivateURLs:resultsTextField scheme:@">>>" color:[NSColor redColor] link:NO];

    //[window makeKeyAndOrderFront:self];
    [NSApp activateIgnoringOtherApps:YES];
    [self doPhase:nfiPhaseDone];
}

// If the user closes the search window, let's just quit
-(BOOL)windowShouldClose:(id)sender
{
    [NSApp terminate:nil];
    return YES;
}

// when first launched, this routine is called when all objects are created
// and initialized.  It's a chance for us to set things up before the user gets
// control of the UI.
-(void)awakeFromNib
{
    findRunning=NO;
    theTask=nil;
    [self doPhase:0];
}

@end
